package hr.seminar;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import hr.genetics.models.Nature;
import hr.genetics.models.Path;
import hr.genetics.models.Vehicle;

public class Program {

	private static int populationSize = 700;
	private static int maxGenerations = 20;
	private static double mutationPossibility = 5.0E-6;
	
	public static void main(String[] args) {
		
		System.out.println("Loading paths...");
		Map<Integer, Path> paths = Path.getPaths();
		System.out.println(paths.size() + " paths loaded.");
		importNewPaths(paths);
		System.out.println(paths.size() + " paths loaded + imported.");
		
		serializePaths(paths);
		
		List<Path> pathsList = new ArrayList<>();
		pathsList.addAll(paths.values());
		
		Nature<Vehicle> nature = new Nature<>(Vehicle.class, populationSize);
		nature.setEnvironment(pathsList.get(0));
		nature.setMutationPossibility(mutationPossibility);
		nature.startEvolution(maxGenerations);
						
	}
		
	public static void importNewPaths(Map<Integer, Path> paths) {
		
		PathScanner ps = new PathScanner();
		List<Path> newPaths = ps.getPaths();
		
		for (Path currPath : newPaths) {
			if (currPath != null)
				paths.put(currPath.getId(), currPath);
		}
		
	}
	
	public static void serializePaths(Map<Integer, Path> paths) {
		
		FileOutputStream fos = null;
		ObjectOutputStream out = null;
		try {
			fos = new FileOutputStream(Paths.get("/Users/mario/workspace/Seminar/data/paths").toFile());
			out = new ObjectOutputStream(fos);
			out.writeObject(paths);
			out.close();    
		} catch(IOException ex) {
			ex.printStackTrace();
		}	
		
	}
	
}
